#ifndef _LINUX_PCSP_H
#define _LINUX_PCSP_H
/*
 * include/linux/pcsp.h
 *
 * /dev/pcsp implementation (dsp & audio for PC Speaker)
 *
 * Copyright (C) 1993-1997  Michael Beck
 */

#include <linux/ioctl.h>

/* MUST BE defined from 0.5 */
#define PCSP_SOUND_VERSION	0x103		/* read 1.03 */


/* card ID, real soundcards use 0 to ... */

#define SNDCARD_PCSP	128
#define SNDCARD_STO1	129
#define SNDCARD_DACM	130
#define SNDCARD_DACS	131
#define SNDCARD_STNC	132


/* IOCTL for changing the play-device, real sample rate etc. */

#define PCSP_SET_DEV					0x00014350
#define PCSP_GET_DEV					0x00024350
#define PCSP_SET_PORTS					0x00034350
#define PCSP_GET_PORTS					0x00044350
#define PCSP_SET_VOL					0x00054350
#define PCSP_GET_VOL					0x00064350
#define PCSP_SET_SRATE					0x00074350
#define PCSP_GET_SRATE					0x00084350
#define PCSP_GET_MEASURE				0x00094350
#define PCSP_SET_EMU_MODE				0x000A4350
#define PCSP_GET_VERSION				0x000F4350

#define PCSP_EMULATION_OFF	0
#define PCSP_EMULATION_ON	1
#define PCSP_EMULATION_QUERY	2


 
#if defined(MODULE) || defined(__KERNEL__)

/* the timer stuff */
#define TIMER_IRQ 0

/* the maximal samplerange for PC-Speaker: 18357 Hz */
#define MIN_CONST	65

#define MAX_SRATE	(CLOCK_TICK_RATE / MIN_CONST)
#define SRATE		(CLOCK_TICK_RATE / pcsp.realrate)

/*
 * the default blocksize for playing thru
 * PC-Speaker, STO1 and STNC use twice as much
 */

#ifndef ABLK_SIZE
#define ABLK_SIZE		16368
#endif

/* the default samplerate for /dev/audio */
#ifndef PCSP_DEFAULT_RATE
#define PCSP_DEFAULT_RATE	8000
#endif

#ifndef PCSP_MIXER
#define PCSP_DEFAULT_LEFT	100
#define PCSP_DEFAULT_RIGHT	100
#endif

/*
 * the "critical" frequency: if the machine is too slow for this, PCSP
 * is disabled
 */
#ifndef PCSP_CRITICAL_FREQ
#define PCSP_CRITICAL_FREQ      12500
#endif

struct pcsp_status {
	int		last_clocks;
	unsigned char	*buf[2];	/* double buffering */
	unsigned char   *buffer;
	unsigned char	*end;
	unsigned	in[2];		/* buffers fill */
	unsigned	xfer;
	unsigned	index;
	unsigned	volume;		/* volume for pc-speaker */
	unsigned	left;		/* left volume */
	unsigned	right;		/* right volume */
	unsigned	srate;		/* sample rate */
	unsigned	si;		/* precalculated step const */
	unsigned	timerC;		/* hardware timer ticks for srate */
	unsigned	timerCF;	/* for fixed samplerate */
	unsigned	act_dev;	/* which device is playing */
	unsigned	port;		/* on which lp-port */
	unsigned	portS;		/* for Stereo */
	unsigned	actual;		/* actual buffer */
	unsigned	realrate;	/* the real sample rate */
	unsigned	maxrate;	/* maximum real sample rate */
	unsigned	audio_fmt;	/* 16 or 8 bit */
	unsigned	fmt_msk;	/* supported data formats */
	unsigned	ablk_size;	/* length of one audio-buffer */
	unsigned	frag_size;	/* length of one audio-fragment */
	unsigned	frag_cnt;	/* number of fragments */
	unsigned char	e;
	char		timer_on;
	char		mode;		/* Mono / Stereo */
	char		is_ulaw;	/* need ULAW->LINEAR */
	char		stereo_emu;	/* set if Stereo is emulated */
	char		enable_emu;	/* set if the emulation is enabled */
	char		first_boot;	/* first time loaded? */
};

/*
 * the globals
 */
extern char pcsp_active, pcsp_speaker, pcsp_enabled;
extern volatile int pcsp_timer0_latch, pcsp_clockticks;
extern struct pcsp_status pcsp;

/* in arch/i386/kernel/irq.c */
extern int pcsp_set_irq(int (*func)(void));
extern int pcsp_release_irq(void);

/* in pcsndrv.c and pcsp_mixer.c */
extern int pcsp_device_init(void);
extern int pcsp_mixer_init(void);

#endif
#endif
